//
// Copyright (c) 2009, Brian Frank and Andy Frank
// Licensed under the Academic Free License version 3.0
//
// History:
//   8 Jan 09   Andy Frank  Creation
//   20 May 09  Andy Frank  Refactor to new OO model
//   8 Jul 09   Andy Frank  Split webappClient into sys/dom
//

fan.dom.ElemPeer = fan.sys.Obj.$extend(fan.sys.Obj);

fan.dom.ElemPeer.prototype.$ctor = function(self) {}

fan.dom.ElemPeer.prototype.tagName = function(self) { return fan.sys.Str.lower(this.elem.nodeName); }

fan.dom.ElemPeer.prototype.id$get = function(self) { return this.elem.id; }
fan.dom.ElemPeer.prototype.id$set = function(self, val) { return this.elem.id = val; }

fan.dom.ElemPeer.prototype.name$get = function(self) { return this.elem.name; }
fan.dom.ElemPeer.prototype.name$set = function(self, val) { return this.elem.name = val; }

fan.dom.ElemPeer.prototype.className$get = function(self) { return this.elem.className; }
fan.dom.ElemPeer.prototype.className$set = function(self, val) { return this.elem.className = val; }

fan.dom.ElemPeer.prototype.hasClassName = function(self, className)
{
  var arr = this.elem.className.split(" ");
  for (var i=0; i<arr.length; i++)
    if (arr[i] == className)
      return true;
  return false;
}

fan.dom.ElemPeer.prototype.addClassName = function(self, className)
{
  if (!this.hasClassName(self, className))
    this.elem.className += " " + className;
  return this;
}

fan.dom.ElemPeer.prototype.removeClassName = function(self, className)
{
  var arr = this.elem.className.split(" ");
  for (var i=0; i<arr.length; i++)
    if (arr[i] == className)
    {
      arr.splice(i, 1);
      break;
    }
  this.elem.className = arr.join(" ");
  return this;
}

fan.dom.ElemPeer.prototype.style = function(self) { return this.elem.style; }
fan.dom.ElemPeer.prototype.computedStyle = function(self)
{
  return (this.elem.currentStyle)
    ? this.elem.currentStyle
    : document.defaultView.getComputedStyle(this.elem, null);
}

fan.dom.ElemPeer.prototype.html$get = function(self) { return this.elem.innerHTML; }
fan.dom.ElemPeer.prototype.html$set = function(self, val) { this.elem.innerHTML = val; }

fan.dom.ElemPeer.prototype.value$get = function(self) { return this.elem.value; }
fan.dom.ElemPeer.prototype.value$set = function(self, val) { this.elem.value = val; }

fan.dom.ElemPeer.prototype.checked$get = function(self) { return this.elem.checked; }
fan.dom.ElemPeer.prototype.checked$set = function(self, val) { this.elem.checked = val; }

fan.dom.ElemPeer.prototype.enabled$get = function(self) { return !this.elem.disabled; }
fan.dom.ElemPeer.prototype.enabled$set = function(self, val) { this.elem.disabled = !val; }

fan.dom.ElemPeer.prototype.get = function(self, name, def)
{
  if (name == "id")      return this.id$get(self);
  if (name == "name")    return this.name$get(self);
  if (name == "class")   return this.className$get(self);
  if (name == "style")   return this.style(self);
  if (name == "value")   return this.value$get(self);
  if (name == "checked") return this.checked$get(self);

  var val = this.elem[name];
  if (val != null) return val;
  if (def != null) return def;
  return null;
}

fan.dom.ElemPeer.prototype.set = function(self, name, val)
{
  if (name == "id")           this.id$set(self, val);
  else if (name == "name")    this.name$set(self, val);
  else if (name == "class")   this.className$set(self, val);
  else if (name == "value")   this.value$set(self, val);
  else if (name == "checked") this.checked$set(self, val);
  else this.elem.setAttribute(name, val);
}

fan.dom.ElemPeer.prototype.x = function(self) { return this.elem.offsetLeft; }
fan.dom.ElemPeer.prototype.y = function(self) { return this.elem.offsetTop; }
fan.dom.ElemPeer.prototype.w = function(self) { return this.elem.offsetWidth; }
fan.dom.ElemPeer.prototype.h = function(self) { return this.elem.offsetHeight; }

fan.dom.ElemPeer.prototype.parent = function(self)
{
  var parent = this.elem.parentNode;
  if (parent == null) return null;
  return fan.dom.ElemPeer.make(parent);
}

fan.dom.ElemPeer.prototype.children = function(self)
{
  var list = new Array();
  var kids = this.elem.childNodes;
  for (var i=0; i<kids.length; i++)
    if (kids[i].nodeType == 1)
      list.push(fan.dom.ElemPeer.make(kids[i]));
  return list;
}

fan.dom.ElemPeer.prototype.first = function(self)
{
  var kids = this.elem.childNodes;
  for (var i=0; i<kids.length; i++)
    if (kids[i].nodeType == 1)
      return fan.dom.ElemPeer.make(kids[i]);
  return null;
}

fan.dom.ElemPeer.prototype.prev = function(self)
{
  var sib = this.elem.previousSibling;
  while (sib != null && sib.nodeType != 1)
    sib = sib.previousSibling;
  if (sib == null) return null;
  return fan.dom.ElemPeer.make(sib);
}

fan.dom.ElemPeer.prototype.next = function(self)
{
  var sib = this.elem.nextSibling;
  while (sib != null && sib.nodeType != 1)
    sib = sib.nextSibling;
  if (sib == null) return null;
  return fan.dom.ElemPeer.make(sib);
}

fan.dom.ElemPeer.prototype.add = function(self, child)
{
  this.elem.appendChild(child.elem);
  return this;
}

fan.dom.ElemPeer.prototype.remove = function(self, child)
{
  this.elem.removeChild(child.elem);
  return this;
}

fan.dom.ElemPeer.prototype.focus = function(self)
{
  // IE throws err if element is not visible, so we need
  // to wrap in a try block
  try { this.elem.focus(); }
  catch (err) {} // ignore
}

fan.dom.ElemPeer.prototype.find = function(self, func)
{
  var kids = this.children(self);
  for (var i=0; i<kids.length; i++)
  {
    var kid = kids[i];
    if (func(kid)) return kid;
    kid = kid.find(func);
    if (kid != null) return kid;
  }
  return null;
}

fan.dom.ElemPeer.prototype.findAll = function(self, func, acc)
{
  if (acc == null) acc = new Array();
  var kids = this.children(self);
  for (var i=0; i<kids.length; i++)
  {
    var kid = kids[i];
    if (func(kid)) acc.push(kid);
    kid.findAll(func, acc);
  }
  return acc;
}

fan.dom.ElemPeer.prototype.onEvent = function(self, type, useCapture, handler)
{
  if (this.elem.addEventListener)
  {
    this.elem.addEventListener(type, function(e) {
      handler(fan.dom.Event.make(e));
    }, useCapture);
  }
  else
  {
    this.elem.attachEvent('on'+type, function(e) {
      handler(fan.dom.Event.make(e));
    });
  }
}

fan.dom.ElemPeer.prototype.effect = function(self)
{
  if (this.fx == null)
  {
    this.fx = fan.dom.Effect.make(self);
    this.fx.peer.sync(self);
  }
  return this.fx;
}
fan.dom.ElemPeer.prototype.fx = null;

fan.dom.ElemPeer.prototype.toStr = function(self)
{
  var name = this.elem.nodeName;
  var type = this.elem.type;
  var id   = this.elem.id;
  var str  = "<" + fan.sys.Str.lower(name);
  if (type != null && type.length > 0) str += " type='" + type + "'";
  if (id != null && id.length > 0) str += " id='" + id + "'"
  str += ">";
  return str;
}

fan.dom.ElemPeer.make = function(elem)
{
  if (elem == null) throw fan.sys.ArgErr.make("elem is null")

  if (elem._fanElem != undefined)
    return elem._fanElem;

  var x = fan.dom.Elem.make();
  x.peer.elem = elem;
  elem._fanElem = x;
  return x;
}